@@ -108,7 +108,8 @@ class Connection(object):
         request_timeout=constants.DEFAULT_REQUEST_TIMEOUT,
         isolation_level=IsolationLevel.AUTOCOMMIT,
         verify=True,
-        http_session=None
+        http_session=None,
+        client_tags=None
     ):
         self.host = host
         self.port = port
@@ -130,6 +131,7 @@ class Connection(object):
         self.redirect_handler = redirect_handler
         self.max_attempts = max_attempts
         self.request_timeout = request_timeout
+        self.client_tags = client_tags
 
         self._isolation_level = isolation_level
         self._request = None
@@ -194,6 +196,7 @@ class Connection(object):
             self.redirect_handler,
             self.max_attempts,
             self.request_timeout,
+            client_tags=self.client_tags
         )
 
     def cursor(self, experimental_python_types=False):
@@ -243,6 +246,12 @@ class Cursor(object):
             return self._query.info_uri
         return None
 
+    @property
+    def update_type(self):
+        if self._query is not None:
+            return self._query.update_type
+        return None
+
     @property
     def description(self):
         if self._query.columns is None:
@@ -385,6 +394,9 @@ class Cursor(object):
         if isinstance(param, list):
             return "ARRAY[%s]" % ','.join(map(self._format_prepared_param, param))
 
+        if isinstance(param, tuple):
+            return "ROW(%s)" % ','.join(map(self._format_prepared_param, param))
+
         if isinstance(param, dict):
             keys = list(param.keys())
             values = [param[key] for key in keys]
@@ -465,7 +477,30 @@ class Cursor(object):
         return result
 
     def executemany(self, operation, seq_of_params):
-        raise trino.exceptions.NotSupportedError
+        """
+        PEP-0249: Prepare a database operation (query or command) and then
+        execute it against all parameter sequences or mappings found in the sequence seq_of_parameters.
+        Modules are free to implement this method using multiple calls to
+        the .execute() method or by using array operations to have the
+        database process the sequence as a whole in one call.
+
+        Use of this method for an operation which produces one or more result
+        sets constitutes undefined behavior, and the implementation is permitted (but not required)
+        to raise an exception when it detects that a result set has been created by an invocation of the operation.
+
+        The same comments as for .execute() also apply accordingly to this method.
+
+        Return values are not defined.
+        """
+        for parameters in seq_of_params[:-1]:
+            self.execute(operation, parameters)
+            self.fetchall()
+            if self._query.update_type is None:
+                raise NotSupportedError("Query must return update type")
+        if seq_of_params:
+            self.execute(operation, seq_of_params[-1])
+        else:
+            self.execute(operation)
 
     def fetchone(self) -> Optional[List[Any]]:
         """
